@@ -6,7 +6,7 @@
 #  Distributed under the terms of the Artistic License 2.0
 #
 #  (c) Richard Poole  2004
-#  (c) David Griffith 2021
+#  (c) David Griffith 2022
 #
 #  Latest version is at https://gitlab.com/DavidGriffith/blorbtools
 # ---------------------------------------------------------------------------
@@ -100,12 +100,14 @@ binmode(BLORB);
 
 read BLORB, $buffer, 12;
 
-my ($groupid, $length, $type) = unpack("NNN", $buffer);
+my ($junk1, $length, $junk2) = unpack("NNN", $buffer);
+
 my $file_length = -s $input_filename;
+my $groupid = substr($buffer, 0, 4);
+my $type = substr($buffer, 8, 4);
 
-if (!$groupid || ($groupid != 0x464F524D)) { die "Not a valid FORM file!\n";}
-
-$type == 0x49465253 or die "Not a Blorb file!\n";
+$groupid eq "FORM" or die "Not a valid FORM file!\n";
+$type eq "IFRS" or die "Not a Blorb file!\n";
 
 print "File length: $file_length bytes\n";
 print "Form length: $length bytes\n";
@@ -239,7 +241,7 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 	if ($options{exec} && ($type eq "ZCOD" or $type eq "GLUL" or
 			$type eq "MAGS")) {
 		$output_filename = "exec_";
-		$output_filename .= sprintf '%0*d', length($execcount) , $execs{$pos};
+		$output_filename .= sprintf '%03d', $execs{$pos};
 		if ($type eq "ZCOD") {
 			$output_filename .= ".z" . unpack("C", $chunkdata);
 		} elsif ($type eq "GLUL") {
@@ -267,7 +269,7 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 			$type eq "MP3 " or $type eq "WAVE" or
 			$type eq "MIDI")) {
 		$output_filename = "snd_";
-		$output_filename .= sprintf '%0*d', length($soundcount) , $sounds{$pos};
+		$output_filename .= sprintf '%03d', $sounds{$pos};
 		if ($type eq "FORM") {
 			$output_filename .= ".aiff";
 			$chunkdata = "FORM" . pack("N", $size) . $chunkdata;
@@ -292,7 +294,7 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 	# Dumping Pict chunks
 	if ($options{images} && ($type eq "PNG " or $type eq "JPEG" or $type eq "GIF " or $type eq "Rect" or $type eq "GFX ")) {
 		$output_filename = "pict_";
-		$output_filename .= sprintf '%0*d', length($imagecount) , $images{$pos};
+		$output_filename .= sprintf '%03d', $images{$pos};
 		if ($type eq "PNG ") {
 			$output_filename .= ".png";
 		} elsif ($type eq "JPEG") {
@@ -304,7 +306,7 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 		} elsif ($type eq "Rect") {
 			my $width = unpack("n", substr($chunkdata, 2));
 			my $height = unpack("n", substr($chunkdata, 6));
-			$output_filename .= ".rect";
+			$output_filename .= ".rec";
 		} else {
 			warn_resource($pos), next;
 		}
@@ -321,7 +323,7 @@ for($pos = 12; $pos < $length; $pos += $size + ($size % 2) + 8) {
 			my($usage, $number, $start) =
 				unpack("a4 NN NN", substr($chunkdata, 0, 12, ""));
 			printf("\t\t%06x: %s %d\n", $start, $usage, $number);
-			$images{$start} = $number if $usage eq "Pict";
+			$images{$start} = $number if $usage eq "Pict" or $usage eq "Rect";
 			$execs{$start} = $number if $usage eq "Exec";
 			$sounds{$start} = $number if $usage eq "Snd ";
 		}
