@@ -15,23 +15,37 @@ INFORM=inform
 LIB=punyinform
 LIBNAME="PunyInform"
 NEWPATH=$LIBPATH/$LIB/lib
-
 ZM_VERSION=-v3
 
-ARGV=$@
-ARGC=0
-for ARG in $@; do
-	ARGC=`expr $ARGC + 1`
+MYNAME=`basename $0 ".sh"`
+
+if [ "$MYNAME" != "$LIB" ] ; then
+	VERSIONSTRING=`echo $MYNAME | sed s/$LIB//g`
+	VERSION=`echo $VERSIONSTRING | sed s/^-//g`
+	INFORM=$INFORM$VERSIONSTRING
+else
+	VERSION=`$INFORM -V | cut -d " " -f2`
+fi
+
+echo "Inform 6 Library Selection Wrapper by David Griffith."
+echo "Using Inform version $VERSION."
+echo "Using $LIBNAME library."
+
+for ARG in "$@"; do
 	case $ARG
 	in
 		-v*)
 			ZM_VERSION=$ARG
-			ARGV=`echo $ARGV | cut -d" " --complement -f $ARGC`
 			break
 		;;
 	esac
-
 done
-echo "Using $LIBNAME library"
-$INFORM $ZM_VERSION +$NEWPATH $ARGV
 
+for ARG in "$@"; do
+	if [ "$ZM_VERSION" != "$ARG" ] ; then
+		LINE="$LINE $ARG"
+	fi
+done
+
+FINAL="$ZM_VERSION +$NEWPATH $LINE"
+$INFORM $FINAL
