@@ -24,7 +24,7 @@ System_file;
 !
 ! A talk topic has the following form:
 !
-! STATUS [ID] TOPIC PLAYERSAYS NPCSAYS [FLAGREF|UNLOCKREF|ROUTINE]*
+! STATUS [ID]* TOPIC PLAYERSAYS NPCSAYS [FLAGREF|UNLOCKREF|ROUTINE|STRING]*
 !
 ! [] = Optional
 ! * = can be more than one
@@ -36,27 +36,45 @@ System_file;
 !   local to the NPC - two different NPCs can use the same ID for different
 !   topics without risk of confusion.
 ! TOPIC is a string or routine for the topic name
-! PLAYERSAYS is a string or routine for what the player says
-! NPCSAYS is a string or routine for what the NPC replies
+! PLAYERSAYS can be on any of these forms (ROUTSTR means a routine or string):
+!    * ROUTSTR
+!    * TM_ADD_BEFORE ROUTSTR ROUTSTR
+!    * TM_ADD_AFTER ROUTSTR ROUTSTR
+!    * TM_ADD_BEFORE_AND_AFTER ROUTSTR ROUTSTR ROUTSTR
+!    I.e. you can add a routine or string to run/print BEFORE the player's 
+!    line, AFTER the player's line, or both. To mute the player's line, give 
+!    it the value TM_NO_LINE
+! NPCSAYS is a string or routine for what the NPC replies. To mute it, give
+!    it the value TM_NO_LINE
 ! FLAGREF is a number 32-299 for a flag to be set (In order to use this,
 !    you must include ext_flags.h before including ext_talk_menu.h)
 ! UNLOCKREF is either a topic ID (300-600) or a relative reference to a topic
 !    (1 to 29) that is activated by this topic. 1 means the next topic,
 !    2 the topic after that etc. The target topic has to have status
 !    TM_INACTIVE (= 0) or TM_ACTIVE (= 30) for this to work. When a topic
-!    is used, it is set to status TM_STALE, and the only way to activate it
-!    when it's stale is to call ReActivateTopic.
+!    is chosen in a conversation, it is set to status TM_STALE, and the only 
+!    way to activate it when it's stale is to call ReActivateTopic.
 ! ROUTINE is a routine to be run. In this routine, the global variable
 !    current_talker refers to the NPC being talked to.
+! STRING is a string to be printed.
+!
+! Whenever a routine is used for PLAYERSAYS, NPCSAYS or ROUTINE, it can set the
+! global talk_menu_talking to false to end the conversation after the current
+! topic. When doing this, you may want to use ROUTINE to print a suitable
+! message about why the conversation ended.
 !
 ! Example of an array giving Linda one active topic (Weather), which will
-! activate the next topic (Heat) and the topic with ID 300 (Herself):
+! activate the next topic (Heat) and the topic with ID 300 (Herself), and if
+! you ask about Heat, she'll just end the conversation without answering.
+!
+! [ EnoughTalk; talk_menu_talking = false; ];
 !
 ! Array talk_array -->
 ! TM_NPC Linda
 ! 0 300 "Herself" "Tell me more about yourself!" "I'm just an average girl."
 ! 30 "Weather" "How do you like the weather?" "It's too hot for me." 1 300
-! 0 "Heat" "Say, don't you like hot weather?" "No, I prefer it cold."
+! 0 "Heat" "Say, don't you like hot weather?" TM_NO_LINE EnoughTalk 
+!          "Linda looks upset and turns away."
 ! TM_NPC 0;
 !
 ! If you find that you need more topic IDs, or more flags, you can define which
@@ -96,7 +114,11 @@ System_file;
 ! will be notified whenever a problem is detected. As usual, use
 ! RUNTIME_ERRORS = 2 to get the maximum amount of information. (This is the
 ! default when compiling in DEBUG mode)
-
+!
+! By default, you can assign the same ID to multiple topics and use this to
+! activate or inactivate multiple topics at once. If you're not using this 
+! option, you can set talk_menu_multi_mode to false, and gain some 
+! performance.
 
 Constant EXT_TALK_MENU = 1;
 
@@ -167,11 +189,35 @@ Constant TM_MSG_EXIT_OPTION "[ENTER] End conversation";
 Constant TM_MSG_PAGE_OPTION "[N] Next page";
 #Endif;
 
+#Ifndef TMPrintLine;
+[TMPrintLine p_actor p_line;
+	! Routine to print a line, by the player or an NPC. Define your own version as needed.
+	if(talk_array-->p_line == TM_NO_LINE) 
+		rfalse;
+	if(p_actor == player)
+		_TMPrintMsg(TM_MSG_YOU, true);
+	else
+		print (The) p_actor;
+	print ": ";
+	_TMCallOrPrint(p_line); ! Can be called as _TMCallOrPrint(p_line, true); if you don't want it to print the newline
+];
+#Endif;
+	
+
+
 Constant TM_INACTIVE 0;
 Constant TM_ACTIVE 30;
 Constant TM_STALE 31;
 Constant TM_NPC -1;
 
+Constant TM_NO_LINE 1; ! Can be used instead of a player's line or an actor's line ONLY!
+Constant TM_ADD_BEFORE 2; ! Can be used directly after subject
+Constant TM_ADD_AFTER 3; ! Can be used directly after subject
+Constant TM_ADD_BEFORE_AND_AFTER 4; ! Can be used directly after subject
+
+Global talk_menu_talking = false;
+Global talk_menu_multi_mode = true;
+
 [ _TMPrintMsg p_msg p_no_newline;
 	if(metaclass(p_msg) == Routine) {
 		p_msg();
@@ -185,7 +231,7 @@ Constant TM_NPC -1;
 	_TMPrintMsg(talk_array-->p_index, p_no_newline);
 ];
 
-[ _SetTopic p_topic p_start p_value _val _find_topic;
+[ _SetTopic p_topic p_start p_value _val _find_topic _base _curr_id _success;
 	! p_topic is 1-29: Act on topic number P_TOPIC, counting from p_start
 	! p_topic is 300-600: Act on topic with ID = P_TOPIC.
 	! p_topic is (-600)-(-300): Act on topic with ID = -P_TOPIC. If p_value is
@@ -219,28 +265,40 @@ Constant TM_NPC -1;
 		p_start++;
 		_val = talk_array-->p_start;
 		if(_val == TM_NPC) {
-			rfalse; ! The topic wasn't found
+			return _success; ! The topic wasn't found, or we are in multi mode
 		}
 		if(_val == TM_INACTIVE or TM_ACTIVE or TM_STALE) {
 			if(_find_topic < 30) {
-				if(_find_topic-- == 1) jump found;
+				if(_find_topic-- == 1) jump _tm_found_topic;
 				continue;
 			}
-			p_start++;
-			if(talk_array-->p_start == _find_topic) {
-				p_start--;
-.found;
-				if(p_value == 1)
-					return talk_array-->p_start;
-				if(_val ~= TM_STALE || p_topic < 0) {
-					talk_array-->p_start = p_value;
-					rtrue; ! Success
+			_base = p_start;
+			while(true) { ! Loop over a list of topic IDs
+				p_start++;
+				_curr_id = talk_array-->p_start;
+				if(_curr_id < TM_FIRST_ID || _curr_id > TM_LAST_ID) {
+					p_start = p_start + 2; ! This is a string or routine, no more IDs here
+					break;
+				}
+				if(_curr_id == _find_topic) {
+					_success = true;
+					p_start = _base;
+._tm_found_topic;
+					if(p_value == 1)
+						return talk_array-->p_start;
+					if(_val ~= TM_STALE || p_topic < 0) {
+						talk_array-->p_start = p_value;
+					}
+					if(_find_topic < 30 || talk_menu_multi_mode == false) {
+						rtrue;
+					}
+					p_start = p_start + 3;
+					break;
+					
 				}
-				rfalse;
 			}
 		}
 	}
-	rfalse; ! Fail
 ];
 
 [ ActivateTopic p_npc p_topic p_start;
@@ -294,7 +352,7 @@ Constant TM_NPC -1;
 
 
 #Ifv5;
-Array TenDashes -> "----------";
+Array TenDashes static -> "----------";
 [ FastDashes p_dashes;
 	while(p_dashes > 10) {
 		@print_table TenDashes 10 1;
@@ -304,6 +362,7 @@ Array TenDashes -> "----------";
 ];
 #Endif;
 
+
 #Ifndef NO_INITTALK;
 [ InitTalk _i _val;
 	! Initialise the conversation system
@@ -311,7 +370,7 @@ Array TenDashes -> "----------";
 	while(true) {
 !		print "Checking #", _i,". ";
 		_val = talk_array-->_i;
-		if(_val == TM_INACTIVE or TM_ACTIVE or TM_STALE) _i = _i + 3;
+		if(_val == TM_INACTIVE or TM_ACTIVE or TM_STALE) _i = _i + 3; ! Only to speed it up
 		else if(_val == TM_NPC) {
 			_i++;
 			_val = talk_array-->_i;
@@ -335,10 +394,11 @@ Array TenDashes -> "----------";
 
 
 #Ifv5;
-[ RunTalk p_npc _i _j _n _val _tm_start _height _width _offset _count _more _has_split;
+[ RunTalk p_npc _i _j _n _val _tm_start _height _width _offset _count _more _has_split _add_msg;
 #Ifnot;
-[ RunTalk p_npc _i _j _n _val _tm_start;
+[ RunTalk p_npc _i _j _n _val _tm_start _add_msg;
 #Endif;
+	talk_menu_talking = true;
 	current_talker = p_npc;
 #Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
 	if(~~(p_npc provides talk_start)) {
@@ -366,13 +426,13 @@ Array TenDashes -> "----------";
 
 	! Print all valid lines to say
 
-.restart_talk_after_line;
+._tm_restart_talk_after_line;
 
 #Ifv5;
 	_offset = 0;
 #Endif;
 
-.restart_talk;
+._tm_restart_talk;
 
 #Ifv5;
 	_count = 0;
@@ -414,7 +474,11 @@ Array TenDashes -> "----------";
 #Endif;
 			_i++;
 			_val = talk_array-->_i;
-			if(_val <= TM_LAST_ID) _i++; ! An ID was found
+			while(_val >= TM_FIRST_ID && _val <= TM_LAST_ID) {
+				! An ID was found
+				_i++;
+				_val = talk_array-->_i;
+			}
 !			print "i:",_i,"!";
 			_TMCallOrPrint(_i);
 		}
@@ -430,7 +494,7 @@ Array TenDashes -> "----------";
 !			"With that, you politely end the conversation.";
 		_TMPrintMsg(_val);
 #Ifv5;
-		jump end_of_talk;
+		jump _tm_end_of_talk;
 #Ifnot;
 		rtrue;
 #Endif;
@@ -471,16 +535,16 @@ Array TenDashes -> "----------";
 			@set_window 0;
 			_TMPrintMsg(TM_MSG_EXIT);
 !			"With that, you politely end the conversation.";
-			jump end_of_talk;
+			jump _tm_end_of_talk;
 		}
 		if(_val == 'n' or 'N' or 130) {
 			if(_more) {
 				_offset = _offset + _height - 6;
-				jump restart_talk;
+				jump _tm_restart_talk;
 			}
 			else if(_offset) {
 				_offset = 0;
-				jump restart_talk;
+				jump _tm_restart_talk;
 			}
 		}
 		_j = _val - 48;
@@ -520,17 +584,31 @@ Array TenDashes -> "----------";
 			talk_array-->_i = TM_STALE;
 			_i++;
 			_val = talk_array-->_i;
-			if(_val < TM_LAST_ID) _i++; ! An ID was found
+			while(_val >= TM_FIRST_ID && _val <= TM_LAST_ID) {
+				! An ID was found
+				_i++;
+				_val = talk_array-->_i;
+			}
 			_i++;
 #Ifv5;
 			@set_window 0;
 #Endif;
-			_TMPrintMsg(TM_MSG_YOU, true);
-			print ": ";
-			_TMCallOrPrint(_i);
+			_add_msg = talk_array-->_i;
+			if(_add_msg == TM_ADD_BEFORE or TM_ADD_AFTER or TM_ADD_BEFORE_AND_AFTER)
+!				_add_msg = 0;
+!			else
+				_i++;
+			if(_add_msg == TM_ADD_BEFORE or TM_ADD_BEFORE_AND_AFTER) {
+				_TMCallOrPrint(_i);
+				_i++;
+			}
+			TMPrintLine(player, _i);
 			_i++;
-			print (The) p_npc, ": ";
-			_TMCallOrPrint(_i);
+			if(_add_msg == TM_ADD_AFTER or TM_ADD_BEFORE_AND_AFTER) {
+				_TMCallOrPrint(_i);
+				_i++;
+			}
+			TMPrintLine(p_npc, _i);
 			break;
 		}
 	}
@@ -547,25 +625,28 @@ Array TenDashes -> "----------";
 			break;
 		}
 
-		if(_val < 30) {
-			! A relative reference to a topic to activate
-			ActivateTopic(p_npc, _val, _j);
-		}
-		#Ifdef EXT_FLAGS;
-		if(_val < TM_FIRST_ID) {
-			! A flag to set
-			SetFlag(_val);
-			continue;
-		}
-		#Endif;
-		if(_val <= TM_LAST_ID) {
-			! An absolute ID for a topic to activate
-			ActivateTopic(p_npc, _val);
-			continue;
+		if(_val > 0) {
+			if(_val < 30) {
+				! A relative reference to a topic to activate
+				ActivateTopic(p_npc, _val, _j);
+				continue;
+			}
+			#Ifdef EXT_FLAGS;
+			if(_val < TM_FIRST_ID) {
+				! A flag to set
+				SetFlag(_val);
+				continue;
+			}
+			#Endif;
+			if(_val <= TM_LAST_ID) {
+				! An absolute ID for a topic to activate
+				ActivateTopic(p_npc, _val);
+				continue;
+			}
 		}
 		#Ifdef DEBUG;
 			#Iftrue RUNTIME_ERRORS > RTE_MINIMUM;
-				if(metaclass(_val) ~= Routine) {
+				if(metaclass(_val) ~= Routine or String) {
 					#Iftrue RUNTIME_ERRORS == RTE_VERBOSE;
 						"ERROR: Talk_menu: Action ", _val, " was not understood for ", (name) p_npc, ".";
 					#Ifnot;
@@ -574,25 +655,29 @@ Array TenDashes -> "----------";
 				}
 			#Endif;
 		#Endif;
-		! A routine to call
-		_TMCallOrPrint(_j, true);
+		! A routine to call or a string to print
+		_TMCallOrPrint(_j);
 	}
 
-	new_line;
-	jump restart_talk_after_line;
-.end_of_talk;
+	if(talk_menu_talking) {
+		new_line;
+		jump _tm_restart_talk_after_line;
+	}
+._tm_end_of_talk;
 #Ifv5;
 	if(_has_split) {
 		@erase_window 1;
 		@split_window 0;
 		@set_window 0;
-		#Ifdef PUNYINFORM_MAJOR_VERSION;
-			statusline_current_height = 0;
-		#Ifnot;
+		#Ifdef TM_NOT_PUNY;
 			gg_statuswin_cursize = 0;
+		#Ifnot;
+			statusline_current_height = 0;
 		#Endif;
 	}
 #Endif;
+	talk_menu_talking = false;
+	rtrue;
 ];
 
 [ TalkSub;
