@@ -2,8 +2,9 @@
 !
 System_file;
 
-Constant PUNYINFORM_MAJOR_VERSION = 3;
-Constant PUNYINFORM_MINOR_VERSION = 5;
+Constant PUNYINFORM_MAJOR_VERSION = 4;
+Constant PUNYINFORM_MINOR_VERSION = 4;
+!Constant PUNYINFORM_VERSION_SUFFIX = "dev"; ! Comment out if none
 
 #Ifndef VN_1636;
 Message fatalerror "*** The PunyInform library needs Inform v6.36 or later to work ***";
@@ -126,6 +127,7 @@ Constant PHASE2 = 2; ! parse the given pattern with side effects
 Constant PHASE2_SUCCESS         = 0;
 Constant PHASE2_ERROR           = 1;
 Constant PHASE2_DISAMBIGUATION  = 2;
+Constant PHASE2_SCOPE           = 3;
 
 
 !Constant WORDSIZE 2; ! set by the compiler from Inform 6.30
@@ -259,6 +261,19 @@ Property when_on            alias initial;
 Property daemon             alias time_out;
 Property article            alias cant_go;
 
+#Ifdef OPTIONAL_LIST_TOGETHER;
+#Ifndef list_together;
+Property list_together;
+#Endif;
+Constant LIST_TOGETHER_PROP_ID = list_together;
+#Endif;
+
+#Ifdef OPTIONAL_ORDERED_TIMERS;
+#Ifndef timer_order;
+Property individual timer_order;
+#Endif;
+#Endif;
+
 Constant FAKE_N_OBJ = 10001;
 Constant FAKE_S_OBJ = 10002;
 Constant FAKE_E_OBJ = 10003;
@@ -314,6 +329,52 @@ Array LanguageNumbers static table
     'sixteen' 16 'seventeen' 17 'eighteen' 18 'nineteen' 19 'twenty' 20;
 #EndIf;
 
+#Ifdef OPTIONAL_LANGUAGE_NUMBER;
+#Ifdef OPTIONAL_ALLOW_WRITTEN_NUMBERS;
+#IfV3;
+Array LanguageNumberStrings static -->
+    "thirteen"
+    "fourteen"
+    "fifteen"
+    "sixteen"
+    "seventeen"
+    "eighteen"
+    "nineteen";
+#Endif;
+#Ifnot; ! Not OPTIONAL_ALLOW_WRITTEN_NUMBERS
+Array LanguageNumberStrings static -->
+	"one"
+	"two"
+	"three"
+    "four"
+    "five"
+    "six"
+    "seven"
+    "eight"
+    "nine"
+    "ten"
+    "eleven"
+    "twelve"
+    "thirteen"
+    "fourteen"
+    "fifteen"
+    "sixteen"
+    "seventeen"
+    "eighteen"
+    "nineteen";
+#Endif; ! Not OPTIONAL_ALLOW_WRITTEN_NUMBERS
+
+Array LanguageNumberTensStrings static -->
+	"twenty"
+    "thirty"
+    "forty"
+    "fifty"
+    "sixty"
+    "seventy"
+    "eighty"
+    "ninety";
+#Endif; ! OPTIONAL_LANGUAGE_NUMBER
+
 
 Constant TT_OBJECT           = 1;    ! one or more words referring to an object
                                      ! it is one of NOUN_TOKEN etc. below
@@ -417,6 +478,7 @@ Global action_to_be;     ! the action being considered
 Global action_reverse;   ! if parameters are in reversed order
 Global meta;             ! if the verb has the meta attribute or not
 Global update_moved;     ! if _NoteObjectAcquisitions should update moved
+Global last_player_child;! keep track of child(player) to run _NoteObjectAcquisitions
 Global verb_word;        ! verb word, eg 'take' in "take all"
 Global verb_wordnum;     ! the position of the verb in the current sentence
 Global consult_from;     ! Word that a "consult" topic starts on
@@ -434,16 +496,20 @@ Global selected_direction_index;
 Global inp1;             ! the same as noun, except when noun is a number
 Global inp2;             ! the same as second, except when second is a number
 Global num_noun_groups;  ! how many noun groups the current sentence contains
+Global parser_phase;     ! the current phase
 Global parser_action;    ! needed for parse_name, ##PluralFound (see DM)
 Global parser_check_multiple;    ! parser should check if multiheld/multiinside
 Global parser_unknown_noun_found;! parser should report unknown word
 Global parser_all_found;! parser encountered 'all'
 Global parser_all_except_object; ! used to filter all but/except patterns
-GLobal parser_one; ! Used to pass extra information from user code to parser
+Global parser_one; ! Used to pass extra information between user code and parser
+Global parser_two; ! Used to pass extra information between user code and parser
 Global usual_grammar_after; ! needed for grammar property parsing
 Global deadflag;
 Global scope_modified;   ! true if daemons, each_turn etc has invalidated scope
 Global scope_objects;
+Global scope_copy_objects;
+Global scope_copy_actor = 0;
 Global scope_routine;    ! for scope=Routine grammar
 Global scope_stage;      ! for scope=Routine grammar: 1, 2 then 3
 Global keep_silent;
@@ -458,9 +524,14 @@ Global inventory_style = 1;
 Global inventory_stage;
 Global phase2_necessary;
 Global receive_action;
-Global scope_copy_actor = 0;
 Global run_after_routines_msg;
 Global run_after_routines_arg_1;
+Global no_implicit_actions;         ! Don't implicitly do things.
+#Ifdef OPTIONAL_MANUAL_SCOPE_BOOST;
+Global react_before_in_scope;
+Global react_after_in_scope;
+Global each_turn_in_scope;
+#Endif;
 
 Global dict_start;
 Global dict_entry_size;
@@ -474,6 +545,7 @@ Global timer1 = 0;
 Global timer2 = 0;
 #EndIf;
 #IfV5;
+Global screen_width;
 Global statusline_current_height = 0;
 Global statusline_height     = 1;
 Global statuswin_current     = false;
@@ -484,8 +556,6 @@ Global clr_bgstatus          = 2;
 Global clr_fgstatus          = 8;
 #endif;
 
-Global visibility_ceiling;
-
 Global normal_directions_enabled = true;
 #Ifdef OPTIONAL_SHIP_DIRECTIONS;
 Global ship_directions_enabled = true;
@@ -498,7 +568,7 @@ Default MAX_WHICH_OBJECTS     10;
 Default MAX_MULTIPLE_OBJECTS  32;
 Default MAX_INPUT_CHARS       78;
 Default MAX_INPUT_WORDS       20;
-Default MAX_SCOPE             32;
+Default MAX_SCOPE             50;
 
 Global cached_scope_pov;        ! Whose POV the cached scope is from (usually the player)
 Global cached_scope_routine;    ! Which scope routine the cached scope is for
@@ -520,14 +590,23 @@ Constant MAX_FLOATING_OBJECTS  32;            ! Max number of objects that have
 #Endif; ! MAX_FLOATING_OBJECTS
 Array floating_objects --> MAX_FLOATING_OBJECTS + 1;
 
+Global c_style = 0;
 Constant WORKFLAG_BIT  $0001;       ! At top level (only), only list objects
                                     ! which have the "workflag" attribute
 Constant ISARE_BIT     $0002;       ! Print " is" or " are" before list
 Constant NEWLINE_BIT   $0004;       ! Print newline after each entry
+Global pc_initial_depth = 0;		! Used to send a depth parameter to PrintContents
 Global pc_indent = 0;				! 0 means PrintContents is not running
+Global pc_skip_next_indent = 0;
+Global pc_depth = 0;				! 0 means PrintContents is not running
+
+#Ifdef OPTIONAL_LIST_TOGETHER;
+Global listing_together; ! The first object# in the group when listing together
+Global lt_value;
+#Endif;
+
 
 Array which_object-->MAX_WHICH_OBJECTS;       ! options for "which book?"
-Array which_level-->MAX_WHICH_OBJECTS;        ! type of which_object
 Array multiple_objects-->MAX_MULTIPLE_OBJECTS;! holds nouns when multi* used
 
 Array buffer->(MAX_INPUT_CHARS + 3);
@@ -590,11 +669,11 @@ Object Directions
 			_w = _parse-->0;
 			_arr = _direction_dict_words;
 			if(normal_directions_enabled) {
-				@scan_table _w _arr (DIRECTION_COUNT * 2) -> _i ?success;
+				@scan_table _w _arr (DIRECTION_COUNT * 2) -> _i ?_matched_word_in_list;
 #Ifndef OPTIONAL_SHIP_DIRECTIONS;
 				if(_w == 'floor' or 'ground') {
 					selected_direction_index = DIRECTION_COUNT - 2;
-	        		jump match2;
+	        		jump _matched_and_have_set_dir_index;
 				}
 #Endif;
 			}
@@ -602,22 +681,22 @@ Object Directions
 #IfDef OPTIONAL_SHIP_DIRECTIONS;
 			_arr = _ship_direction_dict_words;
 			if(ship_directions_enabled)
-				@scan_table _w _arr (DIRECTION_COUNT * 2) -> _i ?success;
+				@scan_table _w _arr (DIRECTION_COUNT * 2) -> _i ?_matched_word_in_list;
 			if((normal_directions_enabled || ship_directions_enabled) &&
 					_w == 'floor' or 'ground') {
 				selected_direction_index = DIRECTION_COUNT - 2;
-        		jump match2;
+        		jump _matched_and_have_set_dir_index;
 			}
 #EndIf;
 			! no match
 !			selected_direction_index = 0;
 !			selected_direction = 0;
 			return 0;
-.success;
+._matched_word_in_list;
 			_i = _i - _arr;
 			@log_shift _i (-1) -> _i; ! Divide by 2
 			selected_direction_index = (_i % DIRECTION_COUNT) + 1;
-.match2;
+._matched_and_have_set_dir_index;
 			selected_direction = direction_properties_array -> selected_direction_index;
 			return 1;
 #IfNot;
@@ -628,38 +707,38 @@ Object Directions
 
 			if(normal_directions_enabled) {
 #Ifndef OPTIONAL_SHIP_DIRECTIONS;
-				@je _w 'out' ?match_out;
-				@je _w 'in' ?match_in;
-				@je _w 'd//' 'down' ?match_d;
-				@je _w 'floor' 'ground' ?match_d;
-				@je _w 'u//' 'up' ?match_u;
+				@je _w 'out' ?_matched_out;
+				@je _w 'in' ?_matched_in;
+				@je _w 'd//' 'down' ?_matched_d;
+				@je _w 'floor' 'ground' ?_matched_d;
+				@je _w 'u//' 'up' ?_matched_u;
 #Endif;
 #Ifdef OPTIONAL_FULL_DIRECTIONS;
-				@je _w 'se' 'southeast' ?match_se;
-				@je _w 'sw' 'southwest' ?match_sw;
-				@je _w 'ne' 'northeast' ?match_ne;
-				@je _w 'nw' 'northwest' ?match_nw;
+				@je _w 'se' 'southeast' ?_matched_se;
+				@je _w 'sw' 'southwest' ?_matched_sw;
+				@je _w 'ne' 'northeast' ?_matched_ne;
+				@je _w 'nw' 'northwest' ?_matched_nw;
 #Endif;
-				@je _w 'e//' 'east' ?match_e;
-				@je _w 'w//' 'west' ?match_w;
-				@je _w 's//' 'south' ?match_s;
-				@je _w 'n//' 'north' ?match_n;
+				@je _w 'e//' 'east' ?_matched_e;
+				@je _w 'w//' 'west' ?_matched_w;
+				@je _w 's//' 'south' ?_matched_s;
+				@je _w 'n//' 'north' ?_matched_n;
 			}
 
 #Ifdef OPTIONAL_SHIP_DIRECTIONS;
 			if(normal_directions_enabled || ship_directions_enabled) {
-				@je _w 'out' ?match_out;
-				@je _w 'in' ?match_in;
-				@je _w 'd//' 'down' ?match_d;
-				@je _w 'floor' 'ground' ?match_d;
-				@je _w 'u//' 'up' ?match_u;
+				@je _w 'out' ?_matched_out;
+				@je _w 'in' ?_matched_in;
+				@je _w 'd//' 'down' ?_matched_d;
+				@je _w 'floor' 'ground' ?_matched_d;
+				@je _w 'u//' 'up' ?_matched_u;
 			}
 
 			if(ship_directions_enabled) {
-				@je _w 'sb' 'starboard' ?match_e;
-				@je _w 'p//' 'port' ?match_w;
-				@je _w 'a//' 'aft' ?match_s;
-				@je _w 'f//' 'fore' ?match_n;
+				@je _w 'sb' 'starboard' ?_matched_e;
+				@je _w 'p//' 'port' ?_matched_w;
+				@je _w 'a//' 'aft' ?_matched_s;
+				@je _w 'f//' 'fore' ?_matched_n;
 			}
 
 #Endif; ! OPTIONAL_SHIP_DIRECTIONS
@@ -668,24 +747,22 @@ Object Directions
 !			selected_direction = 0;
 			return 0;
 
-.match_out; @inc _i;
-.match_in; @inc _i;
-.match_d; @inc _i;
-.match_u; @inc _i;
+._matched_out; @inc _i;
+._matched_in; @inc _i;
+._matched_d; @inc _i;
+._matched_u; @inc _i;
 #Ifdef OPTIONAL_FULL_DIRECTIONS;
-.match_sw; @inc _i;
-.match_se; @inc _i;
-.match_nw; @inc _i;
-.match_ne; @inc _i;
+._matched_sw; @inc _i;
+._matched_se; @inc _i;
+._matched_nw; @inc _i;
+._matched_ne; @inc _i;
 #Endif;
-.match_w; @inc _i;
-.match_e; @inc _i;
-.match_s; @inc _i;
-.match_n; @inc _i;
+._matched_w; @inc _i;
+._matched_e; @inc _i;
+._matched_s; @inc _i;
+._matched_n; @inc _i;
 
-.match;
 			selected_direction_index = _i;
-.match2;
 			selected_direction = direction_properties_array -> selected_direction_index;
 			return 1;
 #EndIf;
@@ -695,3 +772,7 @@ has scenery
 		reactive
 #Endif;
 ;
+
+#Ifdef OPTIONAL_LIST_TOGETHER;
+Object _PunyObj;
+#Endif;
