@@ -5,7 +5,7 @@
 ! ---------------------
 
 Verb 'answer' 'say' 'speak'
-    * topic 'to' creature                       -> Answer;
+	* topic 'to' creature                       -> Answer;
 
 Verb 'ask'
 	* creature 'about' topic                    -> Ask
@@ -14,10 +14,10 @@ Verb 'ask'
 	* 'that' creature topic                     -> AskTo;
 
 Verb 'attack' 'break' 'crack' 'destroy'
-     'fight' 'hit' 'kill' 'murder' 'punch'
-     'smash' 'thump' 'torture' 'wreck'
-    * noun                                      -> Attack
-    * noun 'with' held                          -> Attack;
+		'fight' 'hit' 'kill' 'murder' 'punch'
+		'smash' 'thump' 'torture' 'wreck'
+	* noun                                      -> Attack
+	* noun 'with' held                          -> Attack;
 
 Verb 'climb' 'scale'
 	* noun                                      -> Climb
@@ -38,7 +38,7 @@ Verb 'dig'
 	* noun 'with' held                          -> Dig;
 
 Verb 'drink' 'sip' 'swallow'
-    * noun                                      -> Drink;
+	* noun                                      -> Drink;
 
 Verb 'drop' 'discard' 'throw'
 	* multiheld                                 -> Drop
@@ -46,7 +46,7 @@ Verb 'drop' 'discard' 'throw'
 	* held 'at'/'against'/'on'/'onto' noun      -> ThrowAt;
 
 Verb 'eat'
-    * held                                      -> Eat;
+	* held                                      -> Eat;
 
 #IfDef OPTIONAL_EXTENDED_VERBSET;
 Verb 'enter'
@@ -68,12 +68,12 @@ Verb 'fill'
 	* noun                                      -> Fill;
 
 Verb 'get'
-  * 'out'/'off'/'up' 'of'/'from' noun         -> Exit
-  * 'up'/'out'/'off'                          -> Exit
-  * multi                                     -> Take
-  * 'in'/'into'/'on'/'onto' noun              -> Enter
-  * 'off' noun                                -> GetOff
-  * multiinside 'from'/'off' noun             -> Remove;
+	* 'out'/'off'/'up' 'of'/'from' noun         -> Exit
+	* 'up'/'out'/'off'                          -> Exit
+	* multi                                     -> Take
+	* 'in'/'into'/'on'/'onto' noun              -> Enter
+	* 'off' noun                                -> GetOff
+	* multiinside 'from'/'off' noun             -> Remove;
 
 Verb 'give' 'feed' 'offer' 'pay'
 	* held 'to' creature                        -> Give
@@ -90,7 +90,7 @@ Verb 'go'
 	* noun                                      -> Enter;
 
 Verb 'insert'
-    * multiexcept 'in'/'into' noun              -> Insert;
+	* multiexcept 'in'/'into' noun              -> Insert;
 
 #Ifdef OPTIONAL_FLEXIBLE_INVENTORY;
 Verb 'inventory' 'i//'
@@ -116,6 +116,7 @@ Verb 'lock'
 	* noun 'with' held                          -> Lock;
 
 Verb 'look' 'l//'
+	* noun                                      -> Examine
 	* 'at' noun                                 -> Examine
 	*                                           -> Look
 	* 'in'/'inside'/'on' noun                   -> Search;
@@ -129,17 +130,17 @@ Verb 'pick'
 	* multi 'up'                                -> Take;
 
 Verb 'pull' 'drag'
-    * noun                                      -> Pull;
+	* noun                                      -> Pull;
 
 Verb 'push' 'clear' 'move' 'press' 'shift'
-    * noun                                      -> Push
-    * noun noun=ADirection                      -> PushDir
-    * noun 'to' noun                            -> Transfer;
+	* noun                                      -> Push
+	* noun noun=ADirection                      -> PushDir
+	* noun 'to' noun                            -> Transfer;
 
 Verb 'put' 'place'
 	* multiexcept 'in'/'inside'/'into' noun     -> Insert
 	* multiexcept 'on'/'onto' noun              -> PutOn
-	* 'on' held									-> Wear;
+	* 'on' held                                 -> Wear;
 
 Verb 'read'
 	* noun                                      -> Examine
@@ -161,8 +162,8 @@ Verb 'shed' 'disrobe' 'doff'
 	* held                                      -> Disrobe;
 
 Verb 'shout' 'scream' 'yell'
-    * topic 'to'/'at' creature                  -> Answer
-    * 'to'/'at' noun                            -> ShoutAt
+	* topic 'to'/'at' creature                  -> Answer
+	* 'to'/'at' noun                            -> ShoutAt
 	* topic                                     -> Shout
 	*                                           -> Shout;
 
@@ -218,19 +219,19 @@ Verb 'unlock'
 	* noun 'with' held                          -> Unlock;
 
 Verb 'wait' 'z'
-    *                                           -> Wait;
+	*                                           -> Wait;
 
 Verb 'wear'
-	* held										-> Wear;
+	* held                                      -> Wear;
 
 [ AnswerSub;
 	if (second > 1 && RunLife(second,##Answer) ~= 0) rfalse;
-	PrintMsg(MSG_ANSWER_DEFAULT);
+	return MSG_ANSWER_DEFAULT;
 ];
 
 [ AskSub;
 	if (RunLife(noun,##Ask) ~= 0) rfalse;
-	PrintMsg(MSG_ASK_DEFAULT);
+	return MSG_ASK_DEFAULT;
 ];
 
 [ AskToSub;
@@ -243,14 +244,15 @@ Verb 'wear'
 ];
 
 [ AttackSub;
+	if(ImplicitGrabIfNotHeld(second)) rtrue;
 	if (ObjectIsUntouchable(noun)) return;
 	if (noun has animate && RunLife(noun, ##Attack) ~= 0) rfalse;
-	PrintMsg(MSG_ATTACK_DEFAULT);
+	return MSG_ATTACK_DEFAULT;
 ];
 
 [ ClimbSub;
-	if(noun has animate) { PrintMsg(MSG_CLIMB_ANIMATE); rtrue;}
-	PrintMsg(MSG_CLIMB_DEFAULT);
+	if(noun has animate) return MSG_CLIMB_ANIMATE;
+	return MSG_CLIMB_DEFAULT;
 ];
 
 [ CloseSub;
@@ -264,30 +266,31 @@ Verb 'wear'
 ];
 
 [ ConsultSub;
-	PrintMsg(MSG_CONSULT_NOTHING_INTERESTING);
+	return MSG_CONSULT_NOTHING_INTERESTING;
 ];
 
 [ CutSub;
-	PrintMsg(MSG_CUT_NO_USE);
+	return MSG_CUT_NO_USE;
 ];
 
 [ DigSub;
-	PrintMsg(MSG_DIG_NO_USE);
+	if(ImplicitGrabIfNotHeld(second)) rtrue;
+	return MSG_DIG_NO_USE;
 ];
 
 [ DisrobeSub;
-    if (noun notin player || noun hasnt worn) { PrintMsg(MSG_DISROBE_NOT_WEARING); rtrue; }
-    give noun ~worn;
+	if (noun notin player || noun hasnt worn) return MSG_DISROBE_NOT_WEARING;
+	give noun ~worn;
 	run_after_routines_msg = MSG_DISROBE_DEFAULT;
 ];
 
 [ DrinkSub;
-	PrintMsg(MSG_DRINK_NOTHING_SUITABLE);
+	return MSG_DRINK_NOTHING_SUITABLE;
 ];
 
 [ DropSub _p;
-	if(noun notin player) { PrintMsg(MSG_DROP_NOT_HOLDING); rtrue; }
-	if(_ImplicitDisrobeIfWorn(noun)) rtrue;
+	if(noun notin player) return MSG_DROP_NOT_HOLDING;
+	if(ImplicitDisrobeIfWorn(noun)) rtrue;
 	_p = parent(player);
 	!if(_p ~= location) <<Insert noun _p>>;
 	move noun to parent(player);
@@ -296,9 +299,9 @@ Verb 'wear'
 ];
 
 [ EatSub;
-	if(ObjectIsUntouchable(noun)) return;
-	if(noun has animate) { PrintMsg(MSG_EAT_ANIMATE); rtrue; }
-	if(noun hasnt edible) { PrintMsg(MSG_EAT_INEDIBLE); rtrue; }
+	if(noun has animate) return MSG_EAT_ANIMATE;
+	if(ImplicitGrabIfNotHeld(noun)) rtrue;
+	if(noun hasnt edible) return MSG_EAT_INEDIBLE;
 	remove noun;
 	scope_modified = true;
 	run_after_routines_msg = MSG_EAT_DEFAULT;
@@ -306,6 +309,12 @@ Verb 'wear'
 
 [ EnterSub _door_dir;
 	if(noun has door) {
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+		if(noun.&door_dir == 0) { ! door_to will be checked by Go action anyway
+			_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, noun);
+			rtrue;
+		}
+#EndIf;
 #IfDef OPTIONAL_SIMPLE_DOORS;
 		if(noun.#door_dir > 2) {
 			! This is a Simple Door, where door_dir is an array
@@ -327,9 +336,9 @@ Verb 'wear'
 		<<Go _door_dir>>;
 	}
 	if(noun hasnt enterable) { PrintMsg(MSG_ENTER_YOU_CANT, 'enter'); rtrue; }
-	if(player in noun) { PrintMsg(MSG_ENTER_ALREADY); rtrue; }
+	if(player in noun) return MSG_ENTER_ALREADY;
 	if(noun has container && noun hasnt open) { PrintMsg(MSG_ENTER_NOT_OPEN, noun); rtrue; }
-	if(parent(noun) ~= parent(player)) { PrintMsg(MSG_ENTER_BAD_LOCATION); rtrue; }
+	if(parent(noun) ~= parent(player)) return MSG_ENTER_BAD_LOCATION;
 	PlayerTo(noun, true);
 	run_after_routines_msg = MSG_ENTER_DEFAULT;
 	run_after_routines_arg_1 = 'enter';
@@ -337,23 +346,19 @@ Verb 'wear'
 
 [ ExamineSub x;
 #Ifndef OPTIONAL_NO_DARKNESS;
-	if(location == thedark) {
-		PrintMsg(MSG_EXAMINE_DARK);
-		rtrue;
-	}
+	if(location == thedark) return MSG_EXAMINE_DARK;
 #Endif;
-    if (noun.description == 0) {
-        if (noun has container) {
-            if (noun has open or transparent) <<Search noun>>;
-            else { PrintMsg(MSG_EXAMINE_CLOSED, noun); rtrue;	}
+	if (noun.description == 0) {
+		if (noun has container) {
+			if (noun has open or transparent) <<Search noun>>;
+			else { PrintMsg(MSG_EXAMINE_CLOSED, noun); rtrue;	}
 		}
-        if (noun has switchable) { PrintMsg(MSG_EXAMINE_ONOFF); rtrue; }
-		PrintMsg(MSG_EXAMINE_NOTHING_SPECIAL);
-		rtrue;
-    }
+		if (noun has switchable) return MSG_EXAMINE_ONOFF;
+		return MSG_EXAMINE_NOTHING_SPECIAL;
+	}
 	x = PrintOrRun(noun, description);
-	if (x == 0 && noun has switchable) PrintMsg(MSG_EXAMINE_ONOFF);
 	run_after_routines_msg = 1; ! Run after routines, don't print a msg
+	if (x == 0 && noun has switchable) return MSG_EXAMINE_ONOFF;
 ];
 
 [ ExitSub _p;
@@ -361,15 +366,13 @@ Verb 'wear'
 	if(parent(_p) == 0) {
 		! player not inside, standing in the room
 		if(_p.out_to && noun == 0) <<Go FAKE_OUT_OBJ>>;
-		PrintMsg(MSG_EXIT_ALREADY); 
-		rtrue; 
+		return MSG_EXIT_ALREADY;
 	}
 	if(noun == 0) <<Exit _p>>;
 	if(player notin noun) {
 		if(IndirectlyContains(noun, player)) { PrintMsg(MSG_EXIT_FIRST_LEAVE, parent(player)); rtrue; }
-		if(noun has supporter) { PrintMsg(MSG_EXIT_NOT_ON); rtrue; }
-		PrintMsg(MSG_EXIT_NOT_IN);
-		rtrue;
+		if(noun has supporter) return MSG_EXIT_NOT_ON;
+		return MSG_EXIT_NOT_IN;
 	}
 	if(noun has container && noun hasnt open) { PrintMsg(MSG_EXIT_NOT_OPEN, noun); rtrue; }
 	PlayerTo(parent(noun), true);
@@ -378,30 +381,30 @@ Verb 'wear'
 ];
 
 [ FillSub;
-	PrintMsg(MSG_FILL_NO_WATER);
+	return MSG_FILL_NO_WATER;
 ];
 
 [ GetOffSub;
 	if (parent(player) == noun) <<Exit noun>>;
-	PrintMsg(MSG_EXIT_NOT_ON); rtrue;
+	return MSG_EXIT_NOT_ON;
 ];
 
 [ GiveSub;
 	if(ObjectIsUntouchable(second)) return;
-	if (parent(noun) ~= player) { PrintMsg(MSG_GIVE_NOT_HOLDING); rtrue; }
-	if (second == player)  { PrintMsg(MSG_GIVE_PLAYER); rtrue; }
+	if (noun notin player) return MSG_GIVE_NOT_HOLDING;
+	if (second == player)  return MSG_GIVE_PLAYER;
 	if (RunLife(second, ##Give) ~= 0) rfalse;
-	PrintMsg(MSG_GIVE_DEFAULT);
+	return MSG_GIVE_DEFAULT;
 ];
 
 [ GoSub _prop;
 	! when called Directions have been set properly
 	_prop = selected_direction;
 #IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
-	if(_prop == 0) return RuntimeError(ERR_INVALID_DIR_PROP);
+	if(_prop == 0) { _RunTimeError(ERR_INVALID_DIR_PROP); rtrue; }
 #EndIf;
 
-	GoDir(_prop);
+	return GoDir(_prop);
 ];
 
 ! Generic routine to move object. Can be used for Insert, PutOn, Take, Drop, Transfer, Empty (and Enter/Exit?)
@@ -421,25 +424,21 @@ Verb 'wear'
 	_ancestor = CommonAncestor(noun, second);
 
 	_msg = p_messages-->0;
-	if(_msg && parent(noun) == second) {
-		PrintMsg(_msg);
-		rtrue;
-	}
+	if(_msg && parent(noun) == second) 
+		return _msg;
 	_msg = p_messages-->1;
-	if(_msg && _ancestor == noun) {
-		PrintMsg(_msg);
-		rtrue;
-	}
+	if(_msg && _ancestor == noun) 
+		return _msg;
 	_msg = p_messages-->2;
 	if(_msg && second ~= _ancestor && second has container && second hasnt open) {
 		PrintMsg(_msg, second);
 		rtrue;
 	}
 	_msg = p_messages-->3;
-	if(_msg && noun ~=player && second ~= Directions &&  _ImplicitGrabIfNotHeld(noun)) rtrue;
+	if(_msg && noun ~=player && second ~= Directions &&  ImplicitGrabIfNotHeld(noun)) rtrue;
 
 	_msg = p_messages-->4;
-	if(_msg && _ImplicitDisrobeIfWorn(noun)) rtrue;
+	if(_msg && ImplicitDisrobeIfWorn(noun)) rtrue;
 
 	! run before on receiver
 #IfDef DEBUG;
@@ -451,10 +450,8 @@ Verb 'wear'
 	action = _action;
 
 	_msg = p_messages-->5;
-	if(_msg && second has animate) {
-		PrintMsg(_msg);
-		rtrue;
-	}
+	if(_msg && second has animate)
+		return _msg;
 	_msg = p_messages-->6;
 	if(_msg && second hasnt container) {
 		PrintMsg(_msg, second);
@@ -466,10 +463,8 @@ Verb 'wear'
 		rtrue;
 	}
 	_msg = p_messages-->8;
-	if(_msg && _AtFullCapacity(second)) {
-		PrintMsg(_msg);
-		rtrue;
-	}
+	if(_msg && _AtFullCapacity(second))
+		return _msg;
 
 	move noun to second;
 
@@ -486,7 +481,7 @@ Verb 'wear'
 
 	if (keep_silent) return;
 	_msg = p_messages-->9;
-	if(_msg) PrintMsg(_msg);
+	return _msg; ! Can be 0 or a message#
 ];
 
 
@@ -514,7 +509,7 @@ Array _InsertMessages -->
 
 [ InsertSub;
 	if(noun == player) <<Enter second>>;
-	_MoveNounToSecond(_InsertMessages);
+	return _MoveNounToSecond(_InsertMessages);
 ];
 
 #Ifdef OPTIONAL_FLEXIBLE_INVENTORY;
@@ -525,31 +520,30 @@ Array _InsertMessages -->
 #Ifnot;
 [ InvSub;
 #Endif;
-    if(PrintMsg(MSG_INVENTORY_DEFAULT) == false) {
-		PrintMsg(MSG_INVENTORY_EMPTY);
-	}
 	run_after_routines_msg = 1; ! Run after routines, don't print a msg
+	if(PrintMsg(MSG_INVENTORY_DEFAULT) == false)
+		return MSG_INVENTORY_EMPTY;
 ];
 
 [ JumpSub;
-	PrintMsg(MSG_JUMP);
+	return MSG_JUMP;
 ];
 
 [ JumpOverSub;
-	PrintMsg(MSG_JUMP_OVER);
+	return MSG_JUMP_OVER;
 ];
 
 [ ListenSub;
-    PrintMsg(MSG_LISTEN_DEFAULT);
+	return MSG_LISTEN_DEFAULT;
 ];
 
 [ LockSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (ObjectIsUntouchable(second)) return;
 	if (noun hasnt lockable) { PrintMsg(MSG_LOCK_NOT_A_LOCK, 'lock'); rtrue; }
 	if (noun has locked)  { PrintMsg(MSG_LOCK_ALREADY_LOCKED, 'lock'); rtrue; }
-	if (noun has open) { PrintMsg(MSG_LOCK_CLOSE_FIRST); rtrue; }
-	if (RunRoutines(noun, with_key) ~= second) { PrintMsg(MSG_LOCK_KEY_DOESNT_FIT); rtrue; }
+	if (noun has open) return MSG_LOCK_CLOSE_FIRST;
+	if(ImplicitGrabIfNotHeld(second)) rtrue;
+	if (RunRoutines(noun, with_key) ~= second) return MSG_LOCK_KEY_DOESNT_FIT;
 	give noun locked;
 	run_after_routines_msg = MSG_LOCK_DEFAULT;
 	run_after_routines_arg_1 = 'lock';
@@ -565,8 +559,8 @@ Array _InsertMessages -->
 [ OpenSub;
 	if(ObjectIsUntouchable(noun)) return;
 	if(noun hasnt openable) { PrintMsg(MSG_OPEN_YOU_CANT, 'open'); rtrue; }
-	if(noun has locked) { PrintMsg(MSG_OPEN_LOCKED); rtrue; }
-	if(noun has open) { PrintMsg(MSG_OPEN_ALREADY); rtrue; }
+	if(noun has locked) return MSG_OPEN_LOCKED;
+	if(noun has open) return MSG_OPEN_ALREADY;
 	give noun open;
 	scope_modified = true;
 	run_after_routines_msg = MSG_OPEN_DEFAULT;
@@ -574,22 +568,22 @@ Array _InsertMessages -->
 
 [ PullSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (noun has static)   { PrintMsg(MSG_PULL_STATIC); rtrue; }
-	if (noun has scenery)  { PrintMsg(MSG_PULL_SCENERY); rtrue; }
-	if (noun has animate)  { PrintMsg(MSG_PULL_ANIMATE); rtrue; }
-	PrintMsg(MSG_PULL_DEFAULT); !Nothing obvious happens
+	if (noun has static)   return MSG_PULL_STATIC;
+	if (noun has scenery)  return MSG_PULL_SCENERY;
+	if (noun has animate)  return MSG_PULL_ANIMATE;
+	return MSG_PULL_DEFAULT; !Nothing obvious happens
 ];
 
 [ PushSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (noun has static)   { PrintMsg(MSG_PUSH_STATIC); rtrue; }
-	if (noun has scenery)  { PrintMsg(MSG_PUSH_SCENERY); rtrue; }
-	if (noun has animate)  { PrintMsg(MSG_PUSH_ANIMATE); rtrue; }
-	PrintMsg(MSG_PUSH_DEFAULT);
+	if (noun has static)   return MSG_PUSH_STATIC;
+	if (noun has scenery)  return MSG_PUSH_SCENERY;
+	if (noun has animate)  return MSG_PUSH_ANIMATE;
+	return MSG_PUSH_DEFAULT;
 ];
 
 [ PushDirSub;
-	PrintMsg(MSG_PUSHDIR_DEFAULT);
+	return MSG_PUSHDIR_DEFAULT;
 ];
 
 ! 0: Noun is already in second
@@ -616,13 +610,13 @@ Array _PutOnMessages -->
 
 [ PutOnSub;
 	if(noun == player) <<Enter second>>;
-	_MoveNounToSecond(_PutOnMessages);
+	return _MoveNounToSecond(_PutOnMessages);
 ];
 
 [ RemoveSub _i;
 	_i = parent(noun);
 	if (_i has container && _i hasnt open) { PrintMsg(MSG_REMOVE_CLOSED, _i); rtrue; }
-	if (_i ~= second) { PrintMsg(MSG_REMOVE_NOT_HERE); rtrue; }
+	if (_i ~= second) return MSG_REMOVE_NOT_HERE;
 	if(TryToTakeNoun() ~= false) rtrue;
 	action = ##Remove; if (AfterRoutines()) rtrue;
 	action = ##Take;
@@ -630,66 +624,64 @@ Array _PutOnMessages -->
 ];
 
 [ RubSub;
-	PrintMsg(MSG_RUB_DEFAULT);
+	return MSG_RUB_DEFAULT;
 ];
 
 [ SearchSub _i _plural;
 #Ifndef OPTIONAL_NO_DARKNESS;
-	if(location == thedark) { PrintMsg(MSG_SEARCH_DARK); rtrue; }
+	if(location == thedark) return MSG_SEARCH_DARK;
 #Endif;
 	if (ObjectIsUntouchable(noun)) return;
 	_plural = PrintContents(1, noun);
 
 	if(noun has supporter) {
 		if(_plural == 0)
-			PrintMsg(MSG_SEARCH_NOTHING_ON);
+			return MSG_SEARCH_NOTHING_ON;
 		else
-			PrintMsg(MSG_SEARCH_ON_IT_ISARE);
-		rtrue;
-	}
-	if(noun hasnt container) { PrintMsg(MSG_SEARCH_NOTHING_SPECIAL); rtrue; }
-	if(noun hasnt transparent && noun hasnt open) {
-		PrintMsg(MSG_SEARCH_CANT_SEE_CLOSED); rtrue;
+			return MSG_SEARCH_ON_IT_ISARE;
 	}
+	if(noun hasnt container) return MSG_SEARCH_NOTHING_SPECIAL;
+	if(noun hasnt transparent && noun hasnt open)
+		return MSG_SEARCH_CANT_SEE_CLOSED;
 	if(AfterRoutines()) rtrue;
 	_i = MSG_SEARCH_EMPTY;
 	if(_plural)
 		_i = MSG_SEARCH_IN_IT_ISARE;
-	PrintMsg(_i);
+	return _i;
 ];
 
 [ ShoutSub;
-	PrintMsg(MSG_SHOUT_DEFAULT);
+	return MSG_SHOUT_DEFAULT;
 ];
 
 [ ShoutAtSub;
-	PrintMsg(MSG_SHOUTAT_DEFAULT);
+	return MSG_SHOUTAT_DEFAULT;
 ];
 
 [ ShowSub;
-	if (parent(noun) ~= player) { PrintMsg(MSG_SHOW_NOT_HOLDING); rtrue; }
+	if (parent(noun) ~= player) return MSG_SHOW_NOT_HOLDING;
 	if (second == player) <<Examine noun>>;
 	if (RunLife(second, ##Show) ~= 0) rfalse;
-	PrintMsg(MSG_SHOW_DEFAULT);
+	return MSG_SHOW_DEFAULT;
 ];
 
 [ SmellSub;
 	if(ObjectIsUntouchable(noun)) return;
-	PrintMsg(MSG_SMELL_DEFAULT);
+	return MSG_SMELL_DEFAULT;
 ];
 
 [ SwitchOffSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (noun hasnt switchable) { PrintMsg(MSG_SWITCH_OFF_NOT_SWITCHABLE); rtrue; }
-	if (noun hasnt on)         { PrintMsg(MSG_SWITCH_OFF_NOT_ON); rtrue; }
+	if (noun hasnt switchable) return MSG_SWITCH_OFF_NOT_SWITCHABLE;
+	if (noun hasnt on)         return MSG_SWITCH_OFF_NOT_ON;
 	give noun ~on;
 	run_after_routines_msg = MSG_SWITCH_OFF_DEFAULT;
 ];
 
 [ SwitchOnSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (noun hasnt switchable) { PrintMsg(MSG_SWITCH_ON_NOT_SWITCHABLE); rtrue; }
-	if (noun has on)           { PrintMsg(MSG_SWITCH_ON_ON); rtrue; }
+	if (noun hasnt switchable) return MSG_SWITCH_ON_NOT_SWITCHABLE;
+	if (noun has on)           return MSG_SWITCH_ON_ON;
 	give noun on;
 	run_after_routines_msg = MSG_SWITCH_ON_DEFAULT;
 ];
@@ -700,19 +692,19 @@ Array _PutOnMessages -->
 ];
 
 [ TieSub;
-	PrintMsg(MSG_TIE_DEFAULT);
+	return MSG_TIE_DEFAULT;
 ];
 
 [ TellSub;
-	if (noun == player) { PrintMsg(MSG_TELL_PLAYER); rtrue; }
+	if (noun == player) return MSG_TELL_PLAYER;
 	if (RunLife(noun, ##Tell) ~= 0) rfalse;
-	PrintMsg(MSG_TELL_DEFAULT);
+	return MSG_TELL_DEFAULT;
 ];
 
 [ ThrowAtSub;
-	if(_ImplicitGrabIfNotHeld(noun)) rtrue;
+	if(ImplicitGrabIfNotHeld(noun)) rtrue;
 	if(ObjectIsUntouchable(second)) return;
-	if(_ImplicitDisrobeIfWorn(noun)) rtrue;
+	if(ImplicitDisrobeIfWorn(noun)) rtrue;
 	if(second > 1) {
 #IfDef DEBUG;
 		if(debug_flag & 1) print "(", (name) second, ").before()^";
@@ -721,22 +713,20 @@ Array _PutOnMessages -->
 		if (RunRoutines(second, before) ~= 0) { action = ##ThrowAt; rtrue; }
 		action = ##ThrowAt;
 	}
-	if(second hasnt animate) { PrintMsg(MSG_THROW_ANIMATE); rtrue; }
+	if(second hasnt animate) return MSG_THROW_ANIMATE;
 	if(RunLife(second,##ThrowAt) ~= 0) rfalse;
-	PrintMsg(MSG_THROW_DEFAULT);
+	return MSG_THROW_DEFAULT;
 ];
 
 [ TouchSub;
 	if(ObjectIsUntouchable(noun)) return;
-	PrintMsg(MSG_TOUCH_DEFAULT);
+	return MSG_TOUCH_DEFAULT;
 ];
 
 [ TransferSub;
-	if(noun in second || (noun in parent(player) && selected_direction == d_to)) {
-		PrintMsg(MSG_TRANSFER_ALREADY);
-		rtrue;
-	}
-	if(noun notin player && TryToTakeNoun() == true) return;
+	if(noun in second || (noun in parent(player) && selected_direction == d_to))
+		return MSG_TRANSFER_ALREADY;
+	if(noun notin player && TryToTakeNoun() == true) rtrue;
 	if (second has supporter) <<PutOn noun second>>;
 	if (second == Directions && selected_direction == d_to) <<Drop noun>>;
 	<Insert noun second>;
@@ -744,17 +734,18 @@ Array _PutOnMessages -->
 
 [ TurnSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (noun has static)   { PrintMsg(MSG_TURN_STATIC); rtrue; }
-	if (noun has scenery)  { PrintMsg(MSG_TURN_SCENERY); rtrue; }
-	if (noun has animate)  { PrintMsg(MSG_TURN_ANIMATE); rtrue; }
-	PrintMsg(MSG_TURN_DEFAULT);
+	if (noun has static)   return MSG_TURN_STATIC;
+	if (noun has scenery)  return MSG_TURN_SCENERY;
+	if (noun has animate)  return MSG_TURN_ANIMATE;
+	return MSG_TURN_DEFAULT;
 ];
 
 [ UnlockSub;
 	if (ObjectIsUntouchable(noun)) return;
 	if (noun hasnt lockable) { PrintMsg(MSG_UNLOCK_NOT_A_LOCK, 'unlock'); rtrue; }
 	if (noun hasnt locked)  { PrintMsg(MSG_UNLOCK_ALREADY_UNLOCKED, 'unlock'); rtrue; }
-	if (RunRoutines(noun, with_key) ~= second) { PrintMsg(MSG_UNLOCK_KEY_DOESNT_FIT); rtrue; }
+	if(ImplicitGrabIfNotHeld(second)) rtrue;
+	if (RunRoutines(noun, with_key) ~= second) return MSG_UNLOCK_KEY_DOESNT_FIT;
 	give noun ~locked;
 	run_after_routines_msg = MSG_UNLOCK_DEFAULT;
 	run_after_routines_arg_1 = 'unlock';
@@ -762,12 +753,12 @@ Array _PutOnMessages -->
 
 [ WaitSub;
 	if(AfterRoutines()) rtrue;
-	PrintMsg(MSG_WAIT_DEFAULT);
+	return MSG_WAIT_DEFAULT;
 ];
 
 [ WearSub;
-	if (parent(noun) ~= player) { PrintMsg(MSG_WEAR_NOT_HOLDING); rtrue; }
-	if (noun has worn) { PrintMsg(MSG_WEAR_ALREADY_WORN); rtrue; }
+	if (parent(noun) ~= player) return MSG_WEAR_NOT_HOLDING;
+	if (noun has worn) return MSG_WEAR_ALREADY_WORN;
 	if (noun hasnt clothing) { PrintMsg(MSG_WEAR_NOT_CLOTHING, 'wear'); rtrue; }
 	give noun worn;
 	run_after_routines_msg = MSG_WEAR_DEFAULT;
@@ -808,7 +799,7 @@ Verb 'in' 'inside'
 	*                                           -> GoIn;
 
 Verb 'kiss' 'embrace' 'hug'
-    * creature                                  -> Kiss;
+	* creature                                  -> Kiss;
 
 Verb 'no'
 	*                                           -> No;
@@ -818,7 +809,7 @@ Verb 'peel'
 	* 'off' noun                                -> Take;
 
 Verb 'pray'
-    *                                           -> Pray;
+	*                                           -> Pray;
 
 Verb 'pry' 'prise' 'prize' 'lever' 'jemmy' 'force'
 	* noun 'with' held                          -> Unlock
@@ -862,11 +853,11 @@ Verb 'transfer'
 	* noun 'to' noun                            -> Transfer;
 
 Verb 'wake' 'awake' 'awaken'
-    *                                           -> Wake
-    * 'up'                                      -> Wake
-    * creature                                  -> WakeOther
-    * creature 'up'                             -> WakeOther
-    * 'up' creature                             -> WakeOther;
+	*                                           -> Wake
+	* 'up'                                      -> Wake
+	* creature                                  -> WakeOther
+	* creature 'up'                             -> WakeOther
+	* 'up' creature                             -> WakeOther;
 
 Verb 'wave'
 	*                                           -> WaveHands
@@ -876,15 +867,17 @@ Verb 'yes' 'y//'
 	*                                           -> Yes;
 
 [ BlowSub;
-	PrintMsg(MSG_BLOW_DEFAULT);
+	if(ImplicitGrabIfNotHeld(noun)) rtrue;
+	return MSG_BLOW_DEFAULT;
 ];
 
 [ BurnSub;
-	PrintMsg(MSG_BURN_DEFAULT);
+	if(ImplicitGrabIfNotHeld(second)) rtrue;
+	return MSG_BURN_DEFAULT;
 ];
 
 [ BuySub;
-	PrintMsg(MSG_BUY_DEFAULT);
+	return MSG_BUY_DEFAULT;
 ];
 
 [ EmptySub;
@@ -892,7 +885,7 @@ Verb 'yes' 'y//'
 ];
 
 [ EmptyTSub _i _recipient;
-	if(noun == second) { PrintMsg(MSG_EMPTY_WOULDNT_ACHIEVE); rtrue; }
+	if(noun == second) return MSG_EMPTY_WOULDNT_ACHIEVE;
 	if(noun has container && noun hasnt open) {
 		PrintMsg(MSG_EMPTY_IS_CLOSED, noun);
 		rtrue;
@@ -918,92 +911,92 @@ Verb 'yes' 'y//'
 ];
 
 [ KissSub;
-    if (ObjectIsUntouchable(noun)) return;
-    if (RunLife(noun, ##Kiss) ~= 0) rfalse;
-    if (noun == player) { PrintMsg(MSG_KISS_PLAYER); rtrue; }
-    PrintMsg(MSG_KISS_DEFAULT);
+	if (ObjectIsUntouchable(noun)) return;
+	if (RunLife(noun, ##Kiss) ~= 0) rfalse;
+	if (noun == player) return MSG_KISS_PLAYER;
+	return MSG_KISS_DEFAULT;
 ];
 
 [ MildSub;
-	PrintMsg(MSG_MILD_DEFAULT);
+	return MSG_MILD_DEFAULT;
 ];
 
 [ NoSub;
-	PrintMsg(MSG_RHETORICAL_QUESTION);
+	return MSG_RHETORICAL_QUESTION;
 ];
 
 [ PraySub;
-	PrintMsg(MSG_PRAY_DEFAULT);
+	return MSG_PRAY_DEFAULT;
 ];
 
 [ SetSub;
-	PrintMsg(MSG_SET_DEFAULT);
+	return MSG_SET_DEFAULT;
 ];
 
 [ SetToSub;
-	PrintMsg(MSG_SET_TO_DEFAULT);
+	return MSG_SET_TO_DEFAULT;
 ];
 
 [ SingSub;
-	PrintMsg(MSG_SING_DEFAULT);
+	return MSG_SING_DEFAULT;
 ];
 
 [ SleepSub;
-	PrintMsg(MSG_SLEEP_DEFAULT);
+	return MSG_SLEEP_DEFAULT;
 ];
 
 [ SorrySub;
-	PrintMsg(MSG_SORRY_DEFAULT);
+	return MSG_SORRY_DEFAULT;
 ];
 
 [ StrongSub;
-	PrintMsg(MSG_STRONG_DEFAULT);
+	return MSG_STRONG_DEFAULT;
 ];
 
 [ SqueezeSub;
 	if (ObjectIsUntouchable(noun)) return;
-	if (noun has animate) { PrintMsg(MSG_SQUEEZE_YOURSELF); rtrue; }
-	PrintMsg(MSG_SQUEEZE_DEFAULT);
+	if (noun has animate) return MSG_SQUEEZE_YOURSELF;
+	return MSG_SQUEEZE_DEFAULT;
 ];
 
 [ SwimSub;
-	PrintMsg(MSG_SWIM_DEFAULT);
+	return MSG_SWIM_DEFAULT;
 ];
 
 [ SwingSub;
-	PrintMsg(MSG_SWING_DEFAULT);
+	return MSG_SWING_DEFAULT;
 ];
 
 [ TasteSub;
 	if (ObjectIsUntouchable(noun)) return;
-	PrintMsg(MSG_TASTE_DEFAULT);
+	return MSG_TASTE_DEFAULT;
 ];
 
 [ ThinkSub;
-	PrintMsg(MSG_THINK_DEFAULT);
+	return MSG_THINK_DEFAULT;
 ];
 
 [ WakeSub;
-    PrintMsg(MSG_WAKE_DEFAULT);
+	return MSG_WAKE_DEFAULT;
 ];
 
 [ WakeOtherSub;
-    if (ObjectIsUntouchable(noun)) return;
-    if (RunLife(noun, ##WakeOther) ~= 0) rfalse;
-    PrintMsg(MSG_WAKEOTHER_DEFAULT);
+	if (ObjectIsUntouchable(noun)) return;
+	if (RunLife(noun, ##WakeOther) ~= 0) rfalse;
+	return MSG_WAKEOTHER_DEFAULT;
 ];
 
 [ WaveSub;
-    if(parent(noun) ~= player) { PrintMsg(MSG_WAVE_NOTHOLDING, noun); rtrue; }
-    PrintMsg(MSG_WAVE_DEFAULT);
+	if(parent(noun) ~= player) { PrintMsg(MSG_WAVE_NOTHOLDING, noun); rtrue; }
+	return MSG_WAVE_DEFAULT;
 ];
 
 [ WaveHandsSub;
-	PrintMsg(MSG_WAVEHANDS_DEFAULT);
+	return MSG_WAVEHANDS_DEFAULT;
 ];
 
 [ YesSub;
-	PrintMsg(MSG_RHETORICAL_QUESTION);
+	return MSG_RHETORICAL_QUESTION;
 ];
 
 #EndIf;
@@ -1013,15 +1006,15 @@ Verb 'yes' 'y//'
 ! ---------------------
 
 Verb meta 'again' 'g//'
-    *                                           -> Again;
+	*                                           -> Again;
 
 Verb meta 'brief' 'normal'
 	*                                           -> LookModeNormal;
 
 #Ifdef OPTIONAL_FULL_SCORE;
 Verb meta 'fullscore' 'full'
-    *                                           -> FullScore
-    * 'score'                                   -> FullScore;
+	*                                           -> FullScore
+	* 'score'                                   -> FullScore;
 #Endif;
 
 #Ifndef NO_SCORE;
@@ -1032,8 +1025,8 @@ Verb meta 'notify'
 #Endif;
 
 Verb meta 'oops'
-    *                                           -> Oops
-    * special                                   -> OopsCorrection;
+	*                                           -> Oops
+	* special                                   -> OopsCorrection;
 
 Verb meta 'restart'
 	*                                           -> Restart;
@@ -1045,7 +1038,7 @@ Verb meta 'save'
 	*                                           -> Save;
 
 Verb meta 'score'
-    *                                           -> Score;
+	*                                           -> Score;
 
 Verb meta 'superbrief' 'short'
 	*                                           -> LookModeShort;
@@ -1077,7 +1070,9 @@ Verb meta 'quit' 'q//'
 			_sc = task_scores->(_i);
 			PANum(_sc);
 			_score_sum = _score_sum + _sc;
+#Ifdef PrintTaskName;
 			PrintTaskName(_i);
+#Endif;
 		}
 #IfNot;
 	_i = 0; ! Avoid warning
@@ -1101,39 +1096,39 @@ Verb meta 'quit' 'q//'
 	}
 	@new_line;
 	PANum(score);
-	PrintMsg(MSG_FULLSCORE_END);
+	return MSG_FULLSCORE_END;
 ];
 #EndIf;
 
 [ LookModeNormalSub;
 	lookmode=1;
-	PrintMsg(MSG_LOOKMODE_NORMAL);
+	return MSG_LOOKMODE_NORMAL;
 ];
 
 [ LookModeLongSub;
 	lookmode=2;
-	PrintMsg(MSG_LOOKMODE_LONG);
+	return MSG_LOOKMODE_LONG;
 ];
 
 [ LookModeShortSub;
 	lookmode=3;
-	PrintMsg(MSG_LOOKMODE_SHORT);
+	return MSG_LOOKMODE_SHORT;
 ];
 
 #Ifndef NO_SCORE;
 [ NotifyOnSub;
 	notify_mode = 1;
-	"Score notification on.";
+	return MSG_NOTIFY_ON;
 ];
 
 [ NotifyOffSub;
 	notify_mode = 0;
-	"Score notification off.";
+	return MSG_NOTIFY_OFF;
 ];
 #Endif;
 
 [ OopsSub;
-	"Think nothing of it.";
+	return MSG_OOPS_DEFAULT;
 ];
 
 [ OopsCorrectionSub;
@@ -1148,46 +1143,44 @@ Verb meta 'quit' 'q//'
 ];
 
 [ RestartSub;
-    PrintMsg(MSG_RESTART_CONFIRM);
+	PrintMsg(MSG_RESTART_CONFIRM);
 	if(YesOrNo()) {
 		@restart;
-        PrintMsg(MSG_RESTART_FAILED);
+		return MSG_RESTART_FAILED;
 	}
 ];
 
 #IfV3;
 [ RestoreSub;
-	@restore ?restore_success; ! can't use @restore because of compiler test
+	@restore ?_restore_was_successful; ! can't use @restore because of compiler test
 	verb_word = 'restore';
-	PrintMsg(MSG_RESTORE_FAILED);
-	rtrue;
-.restore_success; ! This is never reached, since a successful restore continues after save opcode.
+	return MSG_RESTORE_FAILED;
+._restore_was_successful; ! This is never reached, since a successful restore continues after save opcode.
 #IfNot;
 [ RestoreSub _flag;
 	@restore -> _flag;
 	! must have failed here so no need to check the flag
-	PrintMsg(MSG_RESTORE_FAILED);
+	return MSG_RESTORE_FAILED;
 #EndIf;
 ];
 
 #IfV3;
 [ SaveSub;
-	@save ?save_success;
-	PrintMsg(MSG_SAVE_FAILED);
-	rtrue;
-.save_success;
-    PrintMsg(MSG_SAVE_DEFAULT);
+	@save ?_save_was_successful;
+	return MSG_SAVE_FAILED;
+._save_was_successful;
+	return MSG_SAVE_DEFAULT;
 #IfNot;
 [ SaveSub _result;
 	@save -> _result;
-	if(_result == 0) { PrintMsg(MSG_SAVE_FAILED); rtrue; }
-	PrintMsg(MSG_SAVE_DEFAULT); ! _result = 1: save ok, 2: Restore ok
+	if(_result == 0) return MSG_SAVE_FAILED;
+	return MSG_SAVE_DEFAULT; ! _result = 1: save ok, 2: Restore ok
 #EndIf;
 ];
 
 #Ifdef NO_SCORE;
 [ ScoreSub;
-	PrintMsg(MSG_SCORE_DEFAULT);
+	return MSG_SCORE_DEFAULT;
 ];
 #Ifnot;
 [ ScoreSub;
@@ -1215,16 +1208,26 @@ Verb meta 'quit' 'q//'
 	print " / Inform v";
 	inversion;
 	print " PunyInform v", PUNYINFORM_MAJOR_VERSION, (char) '.', PUNYINFORM_MINOR_VERSION;
+	if(PUNYINFORM_PATCH_VERSION)
+		print ".", PUNYINFORM_PATCH_VERSION;
+#Ifdef PUNYINFORM_VERSION_SUFFIX;
+	print (string) PUNYINFORM_VERSION_SUFFIX;
+#EndIf;
 	_i = 0;
 #IfDef STRICT_MODE;
 	#IfV5;
-    print " S";
-    _i = 1;
-    #EndIf;
+	print " S";
+	_i = 1;
+	#EndIf;
 #EndIf;
 #IfDef DEBUG;
 	if(_i == 0) print " ";
 	print "D";
+	_i = 1;
+#EndIf;
+#IfTrue RUNTIME_ERRORS > 0;
+	if(_i == 0) print " ";
+	print "R";
 #EndIf;
 	@new_line;
 ];
@@ -1346,12 +1349,10 @@ Verb meta 'verify'
 ];
 
 [ VerifySub;
-	@verify ?Vmaybe;
-	jump Vwrong;
-.Vmaybe;
-	"The game file has verified as intact.";
-.Vwrong;
+	@verify ?_verify_was_maybe_ok;
 	"The game file did not verify as intact, and may be corrupt.";
+._verify_was_maybe_ok;
+	"The game file has verified as intact.";
 ];
 
 
@@ -1363,7 +1364,7 @@ Verb meta 'verify'
 
 #IfDef DEBUG;
 Verb meta 'pronoun' 'pronouns'
-    *                                           -> Pronouns;
+	*                                           -> Pronouns;
 
 Verb meta 'random'
 	*                                           -> RandomSeed
@@ -1375,14 +1376,14 @@ Verb meta 'scope'
 	* noun                                      -> Scope;
 
 Verb meta 'purloin'
-	* noun										-> Purloin;
+	* noun                                      -> Purloin;
 
 Verb meta 'tree'
-	*											-> Tree
-	* noun										-> Tree;
+	*                                           -> Tree
+	* noun                                      -> Tree;
 
 Verb meta 'gonear'
-	* noun										-> GoNear;
+	* noun                                      -> GoNear;
 
 Verb meta 'debug'
 	*                                           -> Debug
@@ -1416,7 +1417,7 @@ Global scope_cnt;
 ];
 
 [ PurloinSub;
-	if(noun == player) { PrintMsg(MSG_TAKE_YOURSELF); rtrue; }
+	if(noun == player) return MSG_TAKE_YOURSELF;
 	if(IndirectlyContains(noun, player)) { PrintMsg(MSG_TAKE_PLAYER_PARENT, noun); rtrue; }
 
 	move noun to player;
@@ -1467,11 +1468,21 @@ Global scope_cnt;
 [TreeSubHelper p_parent p_indent _x _i;
 	objectloop(_x in p_parent) {
 		for(_i = 0 : _i < p_indent : _i++) print "  ";
-		print (name) _x, "^";
+		print (name) _x, " (", _x, ")^";
 		if(child(_x)) TreeSubHelper(_x, p_indent + 1);
 	}
 ];
 
+#Ifdef OPTIONAL_MANUAL_REACTIVE;
+[ MayBeRoutine p_obj p_prop _val;
+	_val = p_obj.&p_prop;
+	if(_val == 0) rfalse;
+	_val = _val-->0;
+	if(_val == 0 or NULL || metaclass(_val) == Routine) rtrue;
+	rfalse;
+];
+#Endif;
+
 [DebugSub _w _o;
 	wn = num_words;
 	_w = NextWord();
@@ -1484,28 +1495,38 @@ Global scope_cnt;
 			print "Probably give reactive to these objects (see notes about ~reactive~ in manual) :^";
 			objectloop(_o) {
 #Ifdef OPTIONAL_REACTIVE_PARSE_NAME;
-				if(_o hasnt reactive && (_o.&react_before ~= 0 ||
-						_o.&react_after ~= 0 || _o.&each_turn ~= 0 ||
-						_o.&add_to_scope ~= 0 || _o.&parse_name ~= 0))
+				if(_o hasnt reactive && (
+						MayBeRoutine(_o, react_before) ||
+						MayBeRoutine(_o, react_after) ||
+						MayBeRoutine(_o, parse_name) ||
+						_o.&add_to_scope ~= 0 ||
+						_o.&each_turn ~= 0))
 					print "(",_o,") ", (name) _o, "^";
 #Ifnot;
-				if(_o hasnt reactive && (_o.&react_before ~= 0 ||
-						_o.&react_after ~= 0 || _o.&each_turn ~= 0 ||
-						_o.&add_to_scope ~= 0))
+				if(_o hasnt reactive && (
+						MayBeRoutine(_o, react_before) ||
+						MayBeRoutine(_o, react_after) ||
+						_o.&add_to_scope ~= 0 ||
+						_o.&each_turn ~= 0))
 					print "(",_o,") ", (name) _o, "^";
 #Endif;
 			}
 			print "^Remove reactive from these objects:^";
 			objectloop(_o) {
 #Ifdef OPTIONAL_REACTIVE_PARSE_NAME;
-				if(_o has reactive && _o.&react_before == 0 &&
-						_o.&react_after == 0 && _o.&each_turn == 0 &&
-						_o.&add_to_scope == 0 && _o.&parse_name == 0)
+				if(_o has reactive && (
+						MayBeRoutine(_o, react_before) == false &&
+						MayBeRoutine(_o, react_after) == false &&
+						MayBeRoutine(_o, parse_name) == false &&
+						_o.&add_to_scope == 0 &&
+						_o.&each_turn == 0))
 					print "(",_o,") ", (name) _o, "^";
 #Ifnot;
-				if(_o has reactive && _o.&react_before == 0 &&
-						_o.&react_after == 0 && _o.&each_turn == 0 &&
-						_o.&add_to_scope == 0)
+				if(_o has reactive && (
+						MayBeRoutine(_o, react_before) == false &&
+						MayBeRoutine(_o, react_after) == false &&
+						_o.&add_to_scope == 0 &&
+						_o.&each_turn == 0))
 					print "(",_o,") ", (name) _o, "^";
 #Endif;
 			}
@@ -1591,7 +1612,7 @@ Global scope_cnt;
 	if(location == thedark) {
 		@new_line;
 		PrintOrRun(location, description);
-		jump EndOfLook;
+		jump _EndOfLookRoutine;
 	}
 #Endif;
 	_obj = parent(player);
@@ -1680,9 +1701,11 @@ Global scope_cnt;
 		! Descend one level
 		_ceil = ScopeCeiling(player, _ceil);
 	} ! while
-.EndOfLook;
+._EndOfLookRoutine;
+#Ifdef LookRoutine;
 	! finally, call the optional library entry routine
-	LookRoutine();
+	RunEntryPointRoutine(LookRoutine);
+#Endif;
 	_action = action; action = ##Look;
 	AfterRoutines();
 	action = _action;
@@ -1700,42 +1723,43 @@ Global scope_cnt;
 #IfDef OPTIONAL_FULL_SCORE;
 #IfDef TASKS_PROVIDED;
 [ Achieved num;
-    if (task_done->num == 0) {
-        task_done->num = 1;
-        score = score + task_scores->num;
-    }
+	if (task_done->num == 0) {
+		task_done->num = 1;
+		score = score + task_scores->num;
+	}
 ];
 #EndIf;
 [ PANum p_m _n;
 	print "  ";
 	_n = p_m;
 	if(_n < 0)    { _n = -p_m; _n = _n*10; }
-	if(_n < 10)   { print "   "; jump Panuml; }
-	if(_n < 100)  { print "  "; jump Panuml; }
+	if(_n < 10)   { print "   "; jump _Panuml; }
+	if(_n < 100)  { print "  "; jump _Panuml; }
 	if(_n < 1000) { print " "; }
-.Panuml;
+._Panuml;
 	print p_m, " ";
 ];
 #EndIf;
 
 [ TryToTakeNoun _i _k _ancestor _after_recipient;
-    ! Try to transfer the given item to the player: return false
-    ! if successful, true if unsuccessful, printing a suitable message
-    ! in the latter case. Return value 2 means it was a success, and a "Taken"
-	! message has been printed.
-    ! People cannot ordinarily be taken.
-    if(noun == player) { PrintMsg(MSG_TAKE_YOURSELF); rtrue; }
+	! Try to transfer the given item to the player. Return values:
+	! 	0: Success, no message has been printed
+	!   1: Failed, a message has been printed
+	!   2: Success, a message has been printed
+
+	! People cannot ordinarily be taken.
+	if(noun == player) { PrintMsg(MSG_TAKE_YOURSELF); rtrue; };
 #Ifdef DisallowTakeAnimate;
 	if(noun has animate && DisallowTakeAnimate(noun)) { PrintMsg(MSG_TAKE_ANIMATE); rtrue; }
 #Ifnot;
-    if(noun has animate) { PrintMsg(MSG_TAKE_ANIMATE); rtrue; }
+	if(noun has animate) { PrintMsg(MSG_TAKE_ANIMATE); rtrue; }
 #Endif;
 	_ancestor = CommonAncestor(player, noun);
 
-    if (_ancestor == 0) {
-        _i = _ObjectScopedBySomething(noun);
-        if (_i) _ancestor = CommonAncestor(player, _i);
-    }
+	if (_ancestor == 0) {
+		_i = _ObjectScopedBySomething(noun);
+		if (_i) _ancestor = CommonAncestor(player, _i);
+	}
 
 	if(noun in player) { PrintMsg(MSG_TAKE_ALREADY_HAVE); rtrue; }
 	if(ObjectIsUntouchable(noun, false, true)) rtrue;
@@ -1743,24 +1767,23 @@ Global scope_cnt;
 
 	! The item is now known to be accessible.
 
-    ! Consult the immediate possessor of the item, if it's in a container
-    ! which the actor is not in.
+	! Consult the immediate possessor of the item, if it's in a container
+	! which the actor is not in.
 
-    _i = parent(noun);
-    if (_i && _i ~= _ancestor && (_i has container or supporter)) {
-        _after_recipient = _i;
+	_i = parent(noun);
+	if (_i && _i ~= _ancestor && (_i has container or supporter)) {
+		_after_recipient = _i;
 #IfDef DEBUG;
 		if(debug_flag & 1) print "(", (name) _i, ").before()^";
 #EndIf;
-        _k = action; action = ##LetGo;
-        if (RunRoutines(_i, before)) { action = _k; rtrue; }
-        action = _k;
-    }
+		_k = action; action = ##LetGo;
+		if (RunRoutines(_i, before)) { action = _k; rtrue; }
+		action = _k;
+	}
 
 	if(noun has scenery) { PrintMsg(MSG_TAKE_SCENERY); rtrue; }
-    if(noun has static) { PrintMsg(MSG_TAKE_STATIC); rtrue; }
-
-    if(_AtFullCapacity(player)) { PrintMsg(MSG_TAKE_NO_CAPACITY); rtrue; }
+	if(noun has static) { PrintMsg(MSG_TAKE_STATIC); rtrue; }
+	if(_AtFullCapacity(player)) { PrintMsg(MSG_TAKE_NO_CAPACITY); rtrue; }
 
 	move noun to player;
 	give noun ~concealed;
@@ -1821,6 +1844,19 @@ Global scope_cnt;
 			_new_location = 0;
 		else {
 			if(_new_location hasnt open) { PrintMsg(MSG_GO_DOOR_CLOSED, _new_location); rtrue; }
+#IfTrue RUNTIME_ERRORS > RTE_MINIMUM;
+#Ifdef OPTIONAL_SIMPLE_DOORS;
+			if(_new_location.&door_dir == 0 || (_new_location.&door_to == 0 && (_new_location.&found_in == 0 || _new_location.#found_in ~= 4))) {
+				_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, _new_location);
+				rtrue;
+			}
+#Ifnot;
+			if(_new_location.&door_to == 0 || _new_location.&door_dir == 0) {
+				_RunTimeError(ERR_OBJECT_HASNT_PROPERTY, _new_location);
+				rtrue;
+			}
+#EndIf;
+#EndIf;
 			_door_to = _new_location.door_to;
 #IfDef OPTIONAL_SIMPLE_DOORS;
 			if(_door_to == 0) {
@@ -1855,8 +1891,7 @@ Global scope_cnt;
 			PrintOrRun(real_location, cant_go);
 			rtrue;
 		}
-        PrintMsg(MSG_GO_CANT_GO);
-		rtrue;
+		return MSG_GO_CANT_GO;
 	}
 
 #IfDef DEBUG;
@@ -1890,8 +1925,10 @@ Global scope_cnt;
 
 
 [ VerbName p_v ;
+#Ifdef PrintVerb;
 	! first give entry routine a chance to override
 	if(PrintVerb(p_v)) return;
+#Endif;
 
 #IfV3;
 	switch(p_v) {
