@@ -11,33 +11,18 @@ System_file;
 #Ifndef MSG_TAKE_YOURSELF;
 Constant MSG_TAKE_YOURSELF "You are always self-possessed.";
 #EndIf;
-#Ifndef MSG_TAKE_SCENERY;
-Constant MSG_TAKE_SCENERY "That's hardly portable.";
-#EndIf;
-#Ifndef MSG_TAKE_STATIC;
-Constant MSG_TAKE_STATIC "That's fixed in place.";
-#EndIf;
 #Ifndef MSG_TAKE_NO_CAPACITY;
 Constant MSG_TAKE_NO_CAPACITY "You are carrying too many things already.";
 #EndIf;
 #Ifndef MSG_TAKE_DEFAULT;
 Constant MSG_TAKE_DEFAULT "Taken.";
 #EndIf;
-#Ifndef MSG_EAT_INEDIBLE;
-Constant MSG_EAT_INEDIBLE "That's plainly inedible.";
-#EndIf;
 #Ifndef MSG_DRINK_NOTHING_SUITABLE;
 Constant MSG_DRINK_NOTHING_SUITABLE "There's nothing suitable to drink here.";
 #EndIf;
 #Ifndef MSG_DROP_DROPPED;
 Constant MSG_DROP_DROPPED "Dropped.";
 #EndIf;
-#Ifndef MSG_OPEN_ALREADY;
-Constant MSG_OPEN_ALREADY "It's already open.";
-#Endif;
-#Ifndef MSG_OPEN_LOCKED;
-Constant MSG_OPEN_LOCKED "It seems to be locked.";
-#Endif;
 #Ifndef MSG_THROW_ANIMATE;
 Constant MSG_THROW_ANIMATE "Futile.";
 #Endif;
@@ -80,9 +65,6 @@ Constant MSG_SAVE_DEFAULT "Ok.";
 #Ifndef MSG_INSERT_ITSELF;
 Constant MSG_INSERT_ITSELF "You can't put something inside itself.";
 #Endif;
-#Ifndef MSG_PUTON_NOT_SUPPORTER;
-Constant MSG_PUTON_NOT_SUPPORTER "You can't put things on top of that.";
-#Endif;
 #Ifndef MSG_PUTON_ITSELF;
 Constant MSG_PUTON_ITSELF "You can't put something on itself.";
 #Endif;
@@ -314,6 +296,14 @@ Default MSG_PUTON_ANIMATE 131;
 Default MSG_LOOKMODE_NORMAL 132;
 Default MSG_LOOKMODE_LONG 133;
 Default MSG_LOOKMODE_SHORT 134;
+Default MSG_AUTO_TAKE_NOT_HELD = 135;
+Default MSG_AUTO_DISROBE_WORN = 136;
+Default MSG_TAKE_SCENERY = 137;
+Default MSG_TAKE_STATIC = 138;
+Default MSG_EAT_INEDIBLE = 139;
+Default MSG_OPEN_ALREADY = 140;
+Default MSG_OPEN_LOCKED = 141;
+Default MSG_PUTON_NOT_SUPPORTER = 142;
 
 #IfDef OPTIONAL_PROVIDE_UNDO_FINAL;
 #Ifndef MSG_UNDO_NOTHING_DONE;
@@ -413,10 +403,12 @@ Constant SKIP_MSG_PUSH_DEFAULT;
 #Iffalse MSG_PUSH_STATIC < 1000;
 #Iffalse MSG_PULL_STATIC < 1000;
 #Iffalse MSG_TURN_STATIC < 1000;
+#Iffalse MSG_TAKE_STATIC < 1000;
 Constant SKIP_MSG_PUSH_STATIC;
 #Endif;
 #Endif;
 #Endif;
+#Endif;
 
 #Iffalse MSG_PUSH_SCENERY < 1000;
 #Iffalse MSG_PULL_SCENERY < 1000;
@@ -450,9 +442,9 @@ Constant SKIP_MSG_DROP_NOT_HOLDING;
 #Iffalse MSG_OPEN_YOU_CANT < 1000;
 #Iffalse MSG_CLOSE_YOU_CANT < 1000;
 #Iffalse MSG_ENTER_YOU_CANT < 1000;
-#Iffalse MSG_LOCK_YOU_CANT < 1000;
-#Iffalse MSG_UNLOCK_YOU_CANT < 1000;
-#Iffalse MSG_WEAR_YOU_CANT < 1000;
+#Iffalse MSG_LOCK_NOT_A_LOCK < 1000;
+#Iffalse MSG_UNLOCK_NOT_A_LOCK < 1000;
+#Iffalse MSG_WEAR_NOT_CLOTHING < 1000;
 Constant SKIP_MSG_OPEN_YOU_CANT;
 #Endif;
 #Endif;
@@ -578,10 +570,12 @@ Constant SKIP_MSG_SWITCH_ON_DEFAULT;
 #Endif;
 
 #Iffalse MSG_PARSER_NOT_HOLDING < 1000;
+#Iffalse MSG_AUTO_TAKE_NOT_HELD < 1000;
 #Iffalse MSG_WAVE_NOTHOLDING < 1000;
 Constant SKIP_MSG_PARSER_NOT_HOLDING;
 #Endif;
 #Endif;
+#Endif;
 
 #Iffalse MSG_CLOSE_NOT_OPEN < 1000;
 #Iffalse MSG_TOUCHABLE_FOUND_CLOSED < 1000;
@@ -648,12 +642,16 @@ Constant SKIP_MSG_EXAMINE_DARK;
 
 	! Not a string, there should be code for the message here
 	switch(p_msg) {
+#Iftrue MSG_TAKE_SCENERY < 1000;
+	MSG_TAKE_SCENERY:
+		print_ret (CTheyreorThats) noun, " hardly portable.";
+#EndIf;
 #Ifndef SKIP_MSG_PUSH_DEFAULT;
 	MSG_PUSH_DEFAULT, MSG_PULL_DEFAULT, MSG_TURN_DEFAULT:
 		"Nothing obvious happens.";
 #Endif;
 #Ifndef SKIP_MSG_PUSH_STATIC;
-	MSG_PUSH_STATIC, MSG_PULL_STATIC, MSG_TURN_STATIC:
+	MSG_PUSH_STATIC, MSG_PULL_STATIC, MSG_TURN_STATIC, MSG_TAKE_STATIC:
 		print_ret (CTheyreorThats) noun, " fixed in place.";
 #Endif;
 #Ifndef SKIP_MSG_PUSH_SCENERY;
@@ -682,8 +680,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 			p_arg_2 = NEWLINE_BIT;
 		}
 		if(PrintContents(p_arg_1, player, p_arg_2)) {
-			if(inventory_style) print (char) '.';
-			new_line;
+			if(inventory_style) print ".^";
 			rtrue;
 		}
 		rfalse;
@@ -705,8 +702,24 @@ Constant SKIP_MSG_EXAMINE_DARK;
 	MSG_OPEN_YOU_CANT, MSG_CLOSE_YOU_CANT, MSG_ENTER_YOU_CANT,
 	MSG_LOCK_NOT_A_LOCK, MSG_UNLOCK_NOT_A_LOCK, MSG_WEAR_NOT_CLOTHING:
 	! p_arg_1 = the base verb for this action ('open', 'close' etc).
-		"That doesn't seem to be something you can ", (verbname) p_arg_1, ".";
+		"You can't ", (verbname) p_arg_1, " ", (ThatorThose) noun, ".";
 #Endif;
+#IfTrue MSG_EAT_INEDIBLE < 1000;
+	MSG_EAT_INEDIBLE:
+		print_ret (CTheyreorThats) noun, " plainly inedible.";
+#EndIf;
+#IfTrue MSG_OPEN_ALREADY < 1000;
+	MSG_OPEN_ALREADY:
+		print_ret (CTheyreorIts) noun, " already open.";
+#EndIf;
+#IfTrue MSG_OPEN_LOCKED < 1000;
+	MSG_OPEN_LOCKED:
+		print_ret (CTheyreorIts) noun, " locked.";
+#EndIf;
+#IfTrue MSG_PUTON_NOT_SUPPORTER < 1000;
+	MSG_PUTON_NOT_SUPPORTER:
+		"You can't put things on top of ", (ThatorThose) second, ".";
+#EndIf;
 #Ifndef SKIP_MSG_TAKE_ANIMATE;
 	MSG_TAKE_ANIMATE, MSG_EAT_ANIMATE:
 		"I don't suppose ", (the) noun, " would care for that.";
@@ -823,6 +836,11 @@ Constant SKIP_MSG_EXAMINE_DARK;
 	! p_arg_1 = the object the player automatically takes off.
 		print "(first taking off ", (the) p_arg_1, ")^";
 #Endif;
+#Iftrue MSG_AUTO_DISROBE_WORN < 1000;
+	MSG_AUTO_DISROBE_WORN:
+	! p_arg_1 = the object the player would need to take off.
+		print "But you would need to take off ", (the) p_arg_1, " first.^";
+#Endif;
 #IfTrue MSG_PARSER_NOTHING_TO_VERB < 1000;
 	MSG_PARSER_NOTHING_TO_VERB:
 	! p_arg_1 = the last word in player input + 1.
@@ -835,7 +853,7 @@ Constant SKIP_MSG_EXAMINE_DARK;
 		}
 #EndIf;
 #Ifndef SKIP_MSG_PARSER_NOT_HOLDING;
-	MSG_PARSER_NOT_HOLDING, MSG_WAVE_NOTHOLDING:
+	MSG_PARSER_NOT_HOLDING, MSG_AUTO_TAKE_NOT_HELD, MSG_WAVE_NOTHOLDING:
 	! p_arg_1 = the object which the player must be holding to perform the
 	! action but isn't.
 		print_ret "But you are not holding ", (the) p_arg_1, ".";
@@ -1113,7 +1131,7 @@ MSG_RUB_DEFAULT, MSG_SQUEEZE_DEFAULT:
 
 default:
 		! No code found. Print an error message.
-		RuntimeError(ERR_UNKNOWN_MSGNO);
+		_RunTimeError(ERR_UNKNOWN_MSGNO);
 	}
 ];
 
@@ -1149,6 +1167,14 @@ default:
 	print "That's";
 ];
 
+[ CTheyreorIts p_obj;
+	if(p_obj ~= player && p_obj hasnt pluralname && p_obj hasnt animate) {
+		print "It's";
+		return;
+	}
+	CTheyreorThats(p_obj);
+];
+
 [OnOff obj;
 	if(obj has on) print "on";
 	else print "off";
@@ -1167,9 +1193,10 @@ Constant ERR_INVALID_DIR_PROP 5;
 Constant ERR_TOO_MANY_FLOATING 6;
 Constant ERR_NOT_DIR_PROP 7;
 Constant ERR_NOT_FAKE_OBJ 8;
+Constant ERR_ILLEGAL_CHOOSEOBJNO 9;
 
-[RunTimeError p_err;
-	print "[Puny error: ";
+[_RunTimeError p_err p_obj _parent;
+	print "[PunyInform error: ";
 	if(p_err ofclass string)
 		print (string) p_err;
 	else {
@@ -1178,7 +1205,7 @@ Constant ERR_NOT_FAKE_OBJ 8;
 		ERR_TOO_MANY_TIMERS_DAEMONS:
 			print "Too many timers/daemons";
 		ERR_OBJECT_HASNT_PROPERTY:
-			print "Object lacks that property";
+			print "Object lacks required property";
 		ERR_SCOPE_FULL:
 			print "Scope full";
 		ERR_UNKNOWN_MSGNO:
@@ -1191,6 +1218,8 @@ Constant ERR_NOT_FAKE_OBJ 8;
 			print "DirPropToFakeObj called with non-dirprop";
 		ERR_NOT_FAKE_OBJ:
 			print "FakeObjToDirProp called with non-fakeobj";
+		ERR_ILLEGAL_CHOOSEOBJNO:
+			print "ChooseObjectFinal_(Pick or Discard) called with nonexistent array index.";
 		default:
 			print "Unknown error (", p_err, ")";
 		}
@@ -1198,5 +1227,14 @@ Constant ERR_NOT_FAKE_OBJ 8;
 		print p_err;
 #EndIf;
 	}
-	"]";
+	print "]^";
+#IfTrue RUNTIME_ERRORS == RTE_VERBOSE;
+	if(p_obj ofclass object) {
+		_parent = parent(p_obj);
+		print "Offending object: ", (the) p_obj, " (", p_obj, ") in ", (name) _parent, " (", _parent, ")^"; 
+	}
+#Ifnot;
+	_parent = p_obj; ! Avoid compiler warning
+#EndIf;
+	rtrue;
 ];
